<?php

namespace Drupal\self_evaluation;

use Drupal\Core\Breadcrumb\Breadcrumb;
use Drupal\Core\Breadcrumb\BreadcrumbBuilderInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Surcharge du breadcrumb.
 */
class BreadcrumbAdminBuilder implements BreadcrumbBuilderInterface {

  use StringTranslationTrait;

  /**
   * The entity_type.manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The request stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $request;

  /**
   * The entity.repository service.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Service path.matcher.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * Constructs a new Breadcrumb.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity_type.manager service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request
   *   The request stack service.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   The entity.repository service.
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   Service path.matcher.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, RequestStack $request, EntityRepositoryInterface $entityRepository, PathMatcherInterface $path_matcher) {
    $this->entityTypeManager = $entity_type_manager;
    $this->request = $request;
    $this->entityRepository = $entityRepository;
    $this->pathMatcher = $path_matcher;
  }

  /**
   * {@inheritdoc}
   */
  public function applies(RouteMatchInterface $route_match): bool {
    $path = $route_match->getRouteObject()->getPath();
    if (substr($path, 0, 23) == '/admin/self-evaluation/') {
      return TRUE;
    }

    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function build(RouteMatchInterface $route_match): Breadcrumb {
    $breadcrumb = new Breadcrumb();
    $breadcrumb->addCacheContexts(['route']);
    $breadcrumb->addLink($this->createLink($this->t('Home'), '<front>'));
    $breadcrumb->addLink($this->createLink($this->t('Administration'), 'system.admin'));
    $breadcrumb->addLink($this->createLink($this->t('Self Evaluation'), 'self_evaluation.admin_self_evaluation'));
    $breadcrumb->addLink($this->createLink($this->t('Content'), 'self_evaluation.admin_self_evaluation.content'));

    if ($route_match->getParameter('self_evaluation') instanceof SelfEvaluationInterface) {
      $self_evaluation = $route_match->getParameter('self_evaluation');
      $this->buildEndBreadcrumb($breadcrumb, $self_evaluation);
    }
    elseif ($route_match->getParameter('self_evaluation_answer') instanceof SelfEvaluationAnswerInterface) {
      /** @var \Drupal\self_evaluation\SelfEvaluationAnswerInterface $self_evaluation_answer */
      $self_evaluation_answer = $route_match->getParameter('self_evaluation_answer');
      $this->buildSelfEvaluationLink($breadcrumb, $self_evaluation_answer->getSelfEvaluation());
      $this->buildThemesListLink($breadcrumb, $self_evaluation_answer->getSelfEvaluation());
      $this->buildEndBreadcrumb($breadcrumb, $self_evaluation_answer);
    }
    elseif ($route_match->getParameter('self_evaluation_theme') instanceof SelfEvaluationThemeInterface) {
      /** @var \Drupal\self_evaluation\SelfEvaluationThemeInterface $self_evaluation_theme */
      $self_evaluation_theme = $route_match->getParameter('self_evaluation_theme');
      $this->buildSelfEvaluationLink($breadcrumb, $self_evaluation_theme->getSelfEvaluation());
      $this->buildThemesListLink($breadcrumb, $self_evaluation_theme->getSelfEvaluation());
      $this->buildEndBreadcrumb($breadcrumb, $self_evaluation_theme);
    }
    elseif ($route_match->getParameter('self_evaluation_theme_threshold') instanceof SelfEvaluationThemeThresholdInterface) {
      /** @var \Drupal\self_evaluation\SelfEvaluationThemeThresholdInterface $self_evaluation_theme_threshold */
      $self_evaluation_theme_threshold = $route_match->getParameter('self_evaluation_theme_threshold');
      $this->commonParts($breadcrumb, $self_evaluation_theme_threshold);
      $breadcrumb->addlink($this->createLink($this->t('Thresholds'), 'entity.self_evaluation_theme.thresholds', ['self_evaluation_theme' => $self_evaluation_theme_threshold->getSelfEvaluationTheme()->id()]));
      $this->buildEndBreadcrumb($breadcrumb, $self_evaluation_theme_threshold);
    }
    elseif ($route_match->getParameter('self_evaluation_question') instanceof SelfEvaluationQuestionInterface) {
      /** @var \Drupal\self_evaluation\SelfEvaluationQuestionInterface $self_evaluation_question */
      $self_evaluation_question = $route_match->getParameter('self_evaluation_question');
      $this->commonParts($breadcrumb, $self_evaluation_question);
      $this->buildQuestionsListLink($breadcrumb, $self_evaluation_question->getSelfEvaluationTheme());
      $this->buildEndBreadcrumb($breadcrumb, $self_evaluation_question);
    }
    elseif ($route_match->getParameter('self_evaluation_question_choice') instanceof SelfEvaluationQuestionChoiceInterface) {
      /** @var \Drupal\self_evaluation\SelfEvaluationQuestionChoiceInterface $self_evaluation_question_choice */
      $self_evaluation_question_choice = $route_match->getParameter('self_evaluation_question_choice');
      $this->buildSelfEvaluationLink($breadcrumb, $self_evaluation_question_choice->getSelfEvaluationQuestion()->getSelfEvaluationTheme()->getSelfEvaluation());
      $this->buildThemesListLink($breadcrumb, $self_evaluation_question_choice->getSelfEvaluationQuestion()->getSelfEvaluationTheme()->getSelfEvaluation());
      $this->buildSelfEvaluationThemeLink($breadcrumb, $self_evaluation_question_choice->getSelfEvaluationQuestion()->getSelfEvaluationTheme());
      $this->buildQuestionsListLink($breadcrumb, $self_evaluation_question_choice->getSelfEvaluationQuestion()->getSelfEvaluationTheme());
      $breadcrumb->addlink($this->createLink(html_entity_decode($self_evaluation_question_choice->getSelfEvaluationQuestion()->label(), ENT_QUOTES, 'UTF-8'), 'entity.self_evaluation_question.edit_form', ['self_evaluation_question' => $self_evaluation_question_choice->getSelfEvaluationQuestion()->id()]));
      $breadcrumb->addlink($this->createLink($this->t('Choices'), 'entity.self_evaluation_question.choices', ['self_evaluation_question' => $self_evaluation_question_choice->getSelfEvaluationQuestion()->id()]));
      $this->buildEndBreadcrumb($breadcrumb, $self_evaluation_question_choice);
    }

    return $breadcrumb;
  }

  /**
   * Build a part of breadcrumb.
   *
   * @param \Drupal\Core\Breadcrumb\Breadcrumb $breadcrumb
   *   Breadcrumb.
   * @param \Drupal\self_evaluation\SelfEvaluationThemableInterface $self_evaluation_themable_entity
   *   A Self evaluation themable entity.
   */
  protected function commonParts(Breadcrumb $breadcrumb, SelfEvaluationThemableInterface $self_evaluation_themable_entity): void {
    $this->buildSelfEvaluationLink($breadcrumb, $self_evaluation_themable_entity->getSelfEvaluationTheme()->getSelfEvaluation());
    $this->buildThemesListLink($breadcrumb, $self_evaluation_themable_entity->getSelfEvaluationTheme()->getSelfEvaluation());
    $this->buildSelfEvaluationThemeLink($breadcrumb, $self_evaluation_themable_entity->getSelfEvaluationTheme());
  }

  /**
   * Build Self Evaluation Link.
   *
   * @param \Drupal\Core\Breadcrumb\Breadcrumb $breadcrumb
   *   Breadcrumb.
   * @param \Drupal\self_evaluation\SelfEvaluationInterface $self_evaluation
   *   A self evaluation entity.
   */
  protected function buildSelfEvaluationLink(Breadcrumb $breadcrumb, SelfEvaluationInterface $self_evaluation) {
    $breadcrumb->addlink($this->createLink(html_entity_decode($self_evaluation->label(), ENT_QUOTES, 'UTF-8'), 'entity.self_evaluation.edit_form', ['self_evaluation' => $self_evaluation->id()]));
  }

  /**
   * Build Self Evaluation Theme Link.
   *
   * @param \Drupal\Core\Breadcrumb\Breadcrumb $breadcrumb
   *   Breadcrumb.
   * @param \Drupal\self_evaluation\SelfEvaluationThemeInterface $self_evaluation_theme
   *   A theme entity.
   */
  protected function buildSelfEvaluationThemeLink(Breadcrumb $breadcrumb, SelfEvaluationThemeInterface $self_evaluation_theme) {
    $breadcrumb->addlink($this->createLink(html_entity_decode($self_evaluation_theme->label(), ENT_QUOTES, 'UTF-8'), 'entity.self_evaluation_theme.edit_form', ['self_evaluation_theme' => $self_evaluation_theme->id()]));
  }

  /**
   * Build Self Evaluation Themes list link.
   *
   * @param \Drupal\Core\Breadcrumb\Breadcrumb $breadcrumb
   *   Breadcrumb.
   * @param \Drupal\self_evaluation\SelfEvaluationInterface $self_evaluation
   *   A self evaluation entity.
   */
  protected function buildThemesListLink(Breadcrumb $breadcrumb, SelfEvaluationInterface $self_evaluation) {
    $breadcrumb->addlink($this->createLink($this->t('Themes and questions'), 'entity.self_evaluation.themes', ['self_evaluation' => $self_evaluation->id()]));
  }

  /**
   * Build Self Evaluation Questions list link.
   *
   * @param \Drupal\Core\Breadcrumb\Breadcrumb $breadcrumb
   *   Breadcrumb.
   * @param \Drupal\self_evaluation\SelfEvaluationThemeInterface $self_evaluation_theme
   *   A theme entity.
   */
  protected function buildQuestionsListLink(Breadcrumb $breadcrumb, SelfEvaluationThemeInterface $self_evaluation_theme) {
    $breadcrumb->addlink($this->createLink($this->t('Questions'), 'entity.self_evaluation_theme.questions', ['self_evaluation_theme' => $self_evaluation_theme->id()]));
  }

  /**
   * Build End breadcrumb.
   *
   * @param \Drupal\Core\Breadcrumb\Breadcrumb $breadcrumb
   *   Breadcrumb.
   * @param \Drupal\Core\Entity\EntityInterface $self_evaluation_type_entity
   *   A self evaluation type entity.
   */
  protected function buildEndBreadcrumb(Breadcrumb $breadcrumb, EntityInterface $self_evaluation_type_entity) {
    $breadcrumb->addlink($this->createLink(html_entity_decode($self_evaluation_type_entity->label(), ENT_QUOTES, 'UTF-8'), '<nolink>'));
  }

  /**
   * Creates a Link object from a given route name and parameters.
   *
   * @param string|array|\Drupal\Component\Render\MarkupInterface $text
   *   The link text.
   * @param string $route_name
   *   The name of the route.
   * @param array $route_parameters
   *   (optional) An associative array of parameter names and values.
   * @param array $options
   *   The options parameter.
   *
   * @return \Drupal\Core\Link
   *   A Generated Link.
   */
  protected function createLink($text, string $route_name, array $route_parameters = [], array $options = []): Link {
    return new Link($text, new Url($route_name, $route_parameters, $options));
  }

}
