<?php

namespace Drupal\self_evaluation\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\Theme\ThemeAccessCheck;
use Drupal\Core\Url;
use Drupal\self_evaluation\SelfEvaluationQuestionChoiceInterface;
use Drupal\system\SystemManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for self evaluation admin routes.
 */
class SelfEvaluationAdminController extends ControllerBase {

  /**
   * System Manager Service.
   *
   * @var \Drupal\system\SystemManager
   */
  protected $systemManager;

  /**
   * The theme access checker service.
   *
   * @var \Drupal\Core\Theme\ThemeAccessCheck
   */
  protected $themeAccess;

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The theme handler service.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   * The menu link tree service.
   *
   * @var \Drupal\Core\Menu\MenuLinkTreeInterface
   */
  protected $menuLinkTree;

  /**
   * Constructs a new SelfEvaluationAdminController.
   *
   * @param \Drupal\system\SystemManager $systemManager
   *   System manager service.
   * @param \Drupal\Core\Theme\ThemeAccessCheck $theme_access
   *   The theme access checker service.
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   * @param \Drupal\Core\Extension\ThemeHandlerInterface $theme_handler
   *   The theme handler.
   * @param \Drupal\Core\Menu\MenuLinkTreeInterface $menu_link_tree
   *   The menu link tree service.
   */
  public function __construct(SystemManager $systemManager, ThemeAccessCheck $theme_access, FormBuilderInterface $form_builder, ThemeHandlerInterface $theme_handler, MenuLinkTreeInterface $menu_link_tree) {
    $this->systemManager = $systemManager;
    $this->themeAccess = $theme_access;
    $this->formBuilder = $form_builder;
    $this->themeHandler = $theme_handler;
    $this->menuLinkTree = $menu_link_tree;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('system.manager'),
      $container->get('access_check.theme'),
      $container->get('form_builder'),
      $container->get('theme_handler'),
      $container->get('menu.link_tree')
    );
  }

  /**
   * Gets a renderable array of admin block links.
   *
   * @return array
   *   The block links renderable array.
   */
  public function adminPage(): array {
    $links = [];

    $links['self_evaluations_content'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Content'),
      'url' => Url::fromRoute('self_evaluation.admin_self_evaluation.content'),
      'description' => $this->t('Manage Self Evaluation entities.'),
    ];

    $links['self_evaluation_configurations'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Configuration'),
      'url' => Url::fromRoute('self_evaluation.admin_self_evaluation.config'),
      'description' => $this->t('Manage Self Evaluation configurations through groupment entities.'),
    ];

    return [
      '#theme' => 'admin_block_content',
      '#content' => $links,
    ];
  }

  /**
   * Gets a renderable array of admin block links.
   *
   * @return array
   *   The block links renderable array.
   */
  public function adminContentPage(): array {
    $links = [];

    $links['self_evaluations'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Self Evaluations'),
      'url' => Url::fromRoute('entity.self_evaluation.collection'),
      'description' => $this->t('Manage Self Evaluation entities.'),
    ];

    $links['self_evaluations_groupments'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Groupments'),
      'url' => Url::fromRoute('entity.self_evaluation_groupment.collection'),
      'description' => $this->t('Manage Self Evaluation Groupments entities.'),
    ];

    return [
      '#theme' => 'admin_block_content',
      '#content' => $links,
    ];
  }

  /**
   * Gets a renderable array of admin block links.
   *
   * @return array
   *   The block links renderable array.
   */
  public function adminConfigPage(): array {
    $links = [];

    $links['self_evaluations_structure'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Structure'),
      'url' => Url::fromRoute('self_evaluation.admin_self_evaluation.config.structure'),
      'description' => $this->t('Manage Self Evaluation entities structure.'),
    ];

    $links['self_evaluations_groupments'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Groupments'),
      'url' => Url::fromRoute('self_evaluation.admin_self_evaluation.config.groupments'),
      'description' => $this->t('Manage Self Evaluation groupment entities.'),
    ];

    return [
      '#theme' => 'admin_block_content',
      '#content' => $links,
    ];
  }

  /**
   * Gets a renderable array of admin block links.
   *
   * @return array
   *   The block links renderable array.
   */
  public function adminStructurePage(): array {
    $links = [];

    $links['self_evaluations_structure_answer'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Answer'),
      'url' => Url::fromRoute('entity.self_evaluation_answer.settings'),
      'description' => $this->t('Manage Self Evaluation Answer Entity.'),
    ];
    $links['self_evenatluations_structure_groupment'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Groupment'),
      'url' => Url::fromRoute('entity.self_evaluation_groupment.settings'),
      'description' => $this->t('Manage Self Evaluation Groupment Entity.'),
    ];
    $links['self_evenatluations_structure_question_choice'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Question Choice'),
      'url' => Url::fromRoute('entity.self_evaluation_question_choice.settings'),
      'description' => $this->t('Manage Self Evaluation Question Choice Entity.'),
    ];
    $links['self_evenatluations_structure_question'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Question'),
      'url' => Url::fromRoute('entity.self_evaluation_question.settings'),
      'description' => $this->t('Manage Self Evaluation Question Entity.'),
    ];
    $links['self_evenatluations_structure_reminder'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Reminder'),
      'url' => Url::fromRoute('entity.self_evaluation_reminder.settings'),
      'description' => $this->t('Manage Self Evaluation Reminder Entity.'),
    ];
    $links['self_evenatluations_structure_self_evaluation'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Self Evaluation'),
      'url' => Url::fromRoute('entity.self_evaluation.settings'),
      'description' => $this->t('Manage Self Evaluation Entity.'),
    ];
    $links['self_evenatluations_structure_theme_threshold'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Theme Threshold'),
      'url' => Url::fromRoute('entity.self_evaluation_theme_threshold.settings'),
      'description' => $this->t('Manage Self Evaluation Theme Threshold Entity.'),
    ];
    $links['self_evenatluations_structure_theme'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Theme'),
      'url' => Url::fromRoute('entity.self_evaluation_theme.settings'),
      'description' => $this->t('Manage Self Evaluation Theme Entity.'),
    ];
    $links['self_evenatluations_structure_user_choices'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('User Choice'),
      'url' => Url::fromRoute('entity.self_evaluation_user_choices.settings'),
      'description' => $this->t('Manage Self Evaluation User Choice Entity.'),
    ];

    return [
      '#theme' => 'admin_block_content',
      '#content' => $links,
    ];
  }

  /**
   * Gets a renderable array of admin block links.
   *
   * @return array
   *   The block links renderable array.
   */
  public function adminGroupmentsPage(): array {
    $links = [];

    $links['self_evaluations_groupment_types'] = [
      'attributes' => ['class' => ['link']],
      'title' => $this->t('Groupments Type'),
      'url' => Url::fromRoute('entity.self_evaluation_groupment_type.collection'),
      'description' => $this->t('Manage Self Evaluation groupment types.'),
    ];

    return [
      '#theme' => 'admin_block_content',
      '#content' => $links,
    ];
  }

  /**
   * Return a custom title for question choice form.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationQuestionChoiceInterface $self_evaluation_question_choice
   *   Current Self Evaluation Question Choice.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   New title.
   */
  public function questionChoiceEditTitle(SelfEvaluationQuestionChoiceInterface $self_evaluation_question_choice): TranslatableMarkup {
    $title = t('Edit Question Choice');
    if (!empty($self_evaluation_question_choice)) {
      $title = t('Edit @question_choice', ['@question_choice' => $self_evaluation_question_choice->getTitleProcessed()]);
    }
    return $title;
  }

}
