<?php

namespace Drupal\self_evaluation\Controller;

use Drupal\Core\Database\Connection;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\self_evaluation\SelfEvaluationInterface;
use Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Returns responses for answers admin routes.
 */
class SelfEvaluationAnswersController extends SelfEvaluationListController {

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * The current Request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The Drupal database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Pager limit.
   *
   * @var int
   */
  protected $limit = 12;

  /**
   * Constructs a new SelfEvaluationAdminController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type manager service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $date_formatter
   *   The date formatter service.
   * @param \Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever
   *   Entity Retriever service.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   * @param \Drupal\Core\Database\Connection $connection
   *   Drupal database connection object.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, DateFormatterInterface $date_formatter, SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever, Request $request, Connection $connection) {
    $this->entityTypeManager = $entity_type_manager;
    $this->dateFormatter = $date_formatter;
    $this->selfEvaluationEntityRetriever = $selfEvaluationEntityRetriever;
    $this->request = $request;
    $this->connection = $connection;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('date.formatter'),
      $container->get('self_evaluation.entity_retriever'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('database')
    );
  }

  /**
   * Return Themes List for a self evaluation.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationInterface $self_evaluation
   *   The current Self Evaluation Entity.
   *
   * @return array
   *   Array return.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function buildList(SelfEvaluationInterface $self_evaluation): array {
    $rows = [];
    $params = [
      'conditions' => [
        [
          'field' => 'self_evaluation',
          'value' => $self_evaluation->id(),
        ],
        [
          'field' => 'token',
          'value' => '',
          'operator' => '<>',
        ],
      ],
      'header_table_sort' => $this->buildHeader(),
      'sorts' => [
        [
          'field' => 'changed',
          'direction' => 'DESC',
        ],
      ],
    ];
    // Total answers.
    $total = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_answer', $params);

    // Total users.
    $select = $this->connection->select('self_evaluation_answer_field_data', 'answer');
    $select->fields('answer', ['uid']);
    $select->distinct();
    $select->condition('self_evaluation', $self_evaluation->id());
    $select->condition('token', '', '<>');
    $users = $select->execute()->fetchCol();

    // Total question user choices.
    $select = $this->connection->select('self_evaluation_user_choices_field_data', 'choices');
    $select->fields('choices', ['id']);
    $select->innerJoin('self_evaluation_answer_field_data', 'answer', 'answer.id = choices.self_evaluation_answer');
    $select->condition('answer.self_evaluation', $self_evaluation->id());
    $select->condition('answer.token', '', '<>');
    $choices = $select->execute()->fetchCol();

    // Post params.
    $request_params = $this->request->request->all();
    if (!empty($request_params['text'])) {
      $query = $this->entityTypeManager()->getStorage('user')->getQuery();
      $query->condition('name', "%{$request_params['text']}%", 'LIKE');
      $uids = $query->execute();

      if (!empty($uids)) {
        $params['conditions'][] = [
          'field' => 'uid',
          'value' => $uids,
          'operator' => 'IN',
        ];
      }
    }
    $answers = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_answer', $params, $this->limit);

    if (!empty($answers)) {
      foreach ($answers as $answer) {
        $rows[] = $this->buildRow($answer);
      }
    }

    $build = $this->render();
    $build['summary'] = [
      '#type' => 'label',
      '#title' => $this->t(
        '@count_answers answer(s), @count_questions answered question(s), @count_users user(s)',
        [
          '@count_answers' => count($total),
          '@count_questions' => count($choices),
          '@count_users' => count($users),
        ]
      ),
      '#weight' => -9,
    ];

    $build['filters'] = [
      '#type' => 'form',
      '#attributes' => [
        'class' => ['table-filter', 'js-show'],
      ],
      '#weight' => -10,
    ];

    $build['filters']['text'] = [
      '#type' => 'search',
      '#title' => $this->t('Search'),
      '#size' => 30,
      '#placeholder' => $this->t('Enter username'),
      '#attributes' => [
        'class' => ['table-filter-text'],
        'name' => 'text',
        'data-table' => '.users-answers-list',
        'autocomplete' => 'off',
        'title' => $this->t('Enter a username to filter by.'),
      ],
    ];

    $build['filters']['actions'] = [
      '#type' => 'actions',
    ];

    $build['filters']['actions']['submit'] = [
      '#type'  => 'submit',
      '#value' => $this->t('Filter'),
    ];

    $build['list']['#attributes']['class'][] = 'users-answers-list';
    $build['list']['#empty'] = $this->t('There are no answer yet.');
    $build['list']['#rows'] = $rows;
    // $build['#attached']['library'][] = 'system/drupal.system.modules';
    if (!empty($this->limit)) {
      $build['pager'] = [
        '#type' => 'pager',
      ];
    }

    return $build;
  }

  /**
   * Builds the header row for the entity listing.
   *
   * @return array
   *   A renderable array structure of header strings.
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildHeader(): array {
    $header = [];

    $header['user'] = [
      'data' => $this->t('User'),
      'field' => 'uid',
      'specifier' => 'uid',
    ];
    $header['token'] = $this->t('Token');
    $header['changed'] = [
      'data' => $this->t('Changed'),
      'field' => 'changed',
      'specifier' => 'changed',
    ];
    $header['shareable'] = $this->t('Shareable');
    $header['results'] = $this->t('Results');
    return $header;
  }

  /**
   * Builds a row for an entity in the entity listing.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for this row of the list.
   *
   * @return array
   *   A renderable array structure of fields for this entity.
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildRow(EntityInterface $entity): array {
    $row = [];
    /** @var \Drupal\self_evaluation\SelfEvaluationAnswerInterface $entity */
    $row['user']['data'] = $entity->getOwner()->label();
    $row['token']['data'] = $entity->getToken();
    $row['changed']['data'] = $this->dateFormatter->format($entity->getChangedTime());
    $row['shareable']['data'] = $entity->isShared() ? t('Yes') : t('No');
    $row['results']['data'] = [
      '#type' => 'link',
      '#title' => $this->t('Results'),
      '#url' => Url::fromRoute('self_evaluation.result_admin', ['self_evaluation_answer' => $entity->id()]),
    ];
    $row['user']['class'][] = 'table-filter-text-source';
    return $row;
  }

}
