<?php

namespace Drupal\self_evaluation\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\self_evaluation\SelfEvaluationQuestionInterface;
use Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for questions choices admin routes.
 */
class SelfEvaluationQuestionChoicesController extends SelfEvaluationListController {

  /**
   * Constructs a new SelfEvaluationAdminController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type manager service.
   * @param \Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever
   *   Entity Retriever service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever) {
    $this->entityTypeManager = $entity_type_manager;
    $this->selfEvaluationEntityRetriever = $selfEvaluationEntityRetriever;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('self_evaluation.entity_retriever')
    );
  }

  /**
   * Return Themes List for a self evaluation.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationQuestionInterface $self_evaluation_question
   *   The current question Entity.
   *
   * @return array
   *   Array return.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function buildList(SelfEvaluationQuestionInterface $self_evaluation_question): array {
    $rows = [];
    $params = [
      'conditions' => [
        [
          'field' => 'self_evaluation_question',
          'value' => $self_evaluation_question->id(),
        ],
      ],
      'sorts' => [
        [
          'field' => 'weight',
        ],
        [
          'field' => 'id',
        ],
      ],
    ];
    $params['header_table_sort'] = $this->buildHeader();
    $choices = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_question_choice', $params);

    if (!empty($choices)) {
      foreach ($choices as $choice) {
        $rows[] = $this->buildRow($choice);
      }
    }
    $build = $this->render();

    $build['add']['#title'] = $this->t('Add a choice');
    $build['add']['#url'] = Url::fromRoute('entity.self_evaluation_question_choice.add_form', [], ['query' => ['self_evaluation_question' => $self_evaluation_question->id()]]);
    $build['list']['#empty'] = $this->t('There are no question choice yet.');
    $build['list']['#rows'] = $rows;
    return $build;
  }

  /**
   * Builds the header row for the entity listing.
   *
   * @return array
   *   A renderable array structure of header strings.
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildHeader(): array {
    $header = [];
    $header['title'] = [
      'data' => $this->t('Title'),
      'field' => 'title',
      'specifier' => 'title',
    ];
    $header['note'] = $this->t('Note');
    $header['weight'] = $this->t('Weight');
    return $header + parent::buildHeader();
  }

  /**
   * Builds a row for an entity in the entity listing.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for this row of the list.
   *
   * @return array
   *   A renderable array structure of fields for this entity.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildRow(EntityInterface $entity): array {
    $row = [];
    /** @var \Drupal\self_evaluation\SelfEvaluationQuestionChoiceInterface $entity */
    $row['title']['data'] = $entity->get('title')->view(['label' => 'hidden']);
    $row['note']['data'] = $entity->getNote();
    $row['weight']['data'] = $entity->get('weight')->value;
    return $row + parent::buildRow($entity);
  }

}
