<?php

namespace Drupal\self_evaluation\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\self_evaluation\SelfEvaluationThemeInterface;
use Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for questions admin routes.
 */
class SelfEvaluationQuestionsController extends SelfEvaluationListController {

  /**
   * Constructs a new SelfEvaluationAdminController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type manager service.
   * @param \Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever
   *   Entity Retriever service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever) {
    $this->entityTypeManager = $entity_type_manager;
    $this->selfEvaluationEntityRetriever = $selfEvaluationEntityRetriever;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('self_evaluation.entity_retriever')
    );
  }

  /**
   * Return questions List for a theme.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationThemeInterface $self_evaluation_theme
   *   The current Theme Entity.
   *
   * @return array
   *   Array return.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function buildList(SelfEvaluationThemeInterface $self_evaluation_theme): array {
    $rows = [];

    $params = [
      'conditions' => [
        [
          'field' => 'self_evaluation_theme',
          'value' => $self_evaluation_theme->id(),
        ],
      ],
      'sorts' => [
        [
          'field' => 'weight',
        ],
      ],
    ];
    $questions = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_question', $params);

    if (!empty($questions)) {
      foreach ($questions as $question) {
        $rows[] = $this->buildRow($question);
      }
    }
    $build = $this->render();

    $build['add']['#title'] = $this->t('Add a question');
    $build['add']['#url'] = Url::fromRoute('entity.self_evaluation_question.add_form', [], ['query' => ['self_evaluation_theme' => $self_evaluation_theme->id()]]);
    $build['list']['#empty'] = $this->t('There are no question yet.');
    $build['list']['#rows'] = $rows;
    return $build;
  }

  /**
   * Builds the header row for the entity listing.
   *
   * @return array
   *   A renderable array structure of header strings.
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildHeader(): array {
    $header = [];
    $header['title'] = $this->t('Title');
    $header['choices'] = $this->t('Choices');
    $header['weight'] = $this->t('Weight');
    $header['status'] = $this->t('Publishing status');
    return $header + parent::buildHeader();
  }

  /**
   * Builds a row for an entity in the entity listing.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for this row of the list.
   *
   * @return array
   *   A renderable array structure of fields for this entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildRow(EntityInterface $entity): array {
    $row = [];
    /** @var \Drupal\self_evaluation\SelfEvaluationQuestionInterface $entity */
    $row['title']['data'] = $entity->label();
    $row['choices']['data'] = $this->getChoices($entity);
    $row['weight']['data'] = $entity->get('weight')->value;
    $row['status']['data'] = $entity->isEnabled() ? $this->t('Published') : $this->t('Unpublished');
    return $row + parent::buildRow($entity);
  }

  /**
   * Get data for choices cell for a question.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Current Entity Question.
   *
   * @return array
   *   Data.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getChoices(EntityInterface $entity): array {
    $params = [
      'conditions' => [
        [
          'field' => 'self_evaluation_question',
          'value' => $entity->id(),
        ],
      ],
    ];
    $choices = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_question_choice', $params);
    if (!empty($choices)) {
      return [
        '#type' => 'link',
        '#title' => $this->t("List of choices"),
        '#url' => Url::fromRoute('entity.self_evaluation_question.choices', ['self_evaluation_question' => $entity->id()]),
      ];
    }
    return [
      '#type' => 'link',
      '#title' => $this->t("Add a choice"),
      '#url' => Url::fromRoute('entity.self_evaluation_question_choice.add_form', [], ['query' => ['self_evaluation_question' => $entity->id()]]),
    ];
  }

}
