<?php

namespace Drupal\self_evaluation\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\self_evaluation\SelfEvaluationInterface;
use Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for themes admin routes.
 */
class SelfEvaluationThemesQuestionsController extends SelfEvaluationListController {

  /**
   * Constructs a new SelfEvaluationThemesQuestionsController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type manager service.
   * @param \Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever
   *   Entity Retriever service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever) {
    $this->entityTypeManager = $entity_type_manager;
    $this->selfEvaluationEntityRetriever = $selfEvaluationEntityRetriever;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('self_evaluation.entity_retriever')
    );
  }

  /**
   * Return Themes List for a self evaluation.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationInterface $self_evaluation
   *   The current Self Evaluation Entity.
   *
   * @return array
   *   Array return.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function buildList(SelfEvaluationInterface $self_evaluation): array {
    $rows = [];
    $params = [
      'conditions' => [
        [
          'field' => 'self_evaluation',
          'value' => $self_evaluation->id(),
        ],
      ],
      'sorts' => [
        [
          'field' => 'weight',
        ],
        [
          'field' => 'name',
        ],
      ],
    ];
    $themes = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_theme', $params);

    if (!empty($themes)) {
      foreach ($themes as $theme) {
        $rows[] = $this->buildRow($theme);
      }
    }
    $build = $this->render();

    $build['add']['#title'] = $this->t('Add a theme');
    $build['add']['#url'] = Url::fromRoute('entity.self_evaluation_theme.add_form', [], ['query' => ['self_evaluation' => $self_evaluation->id()]]);
    $build['list']['#empty'] = $this->t('There are no theme yet.');
    $build['list']['#rows'] = $rows;
    return $build;
  }

  /**
   * Builds the header row for the entity listing.
   *
   * @return array
   *   A renderable array structure of header strings.
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildHeader(): array {
    $header = [];
    $header['title'] = $this->t('Title');
    $header['questions'] = $this->t('Questions');
    $header['thresholds'] = $this->t('Thresholds');
    $header['weight'] = $this->t('Weight');
    $header['status'] = $this->t('Publishing status');
    return $header + parent::buildHeader();
  }

  /**
   * Builds a row for an entity in the entity listing.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for this row of the list.
   *
   * @return array
   *   A renderable array structure of fields for this entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildRow(EntityInterface $entity): array {
    $row = [];
    /** @var \Drupal\self_evaluation\SelfEvaluationThemeInterface $entity */
    $row['title']['data'] = $entity->label();
    $row['questions']['data'] = $this->getQuestions($entity);
    $row['thresholds']['data'] = $this->getThresholdsCell($entity);
    $row['weight']['data'] = $entity->get('weight')->value;
    $row['status']['data'] = $entity->isEnabled() ? $this->t('Published') : $this->t('Unpublished');
    return $row + parent::buildRow($entity);
  }

  /**
   * Get data for questions cell for a theme.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Current Entity Theme.
   *
   * @return array
   *   Data.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getQuestions(EntityInterface $entity): array {
    $params = [
      'conditions' => [
        [
          'field' => 'self_evaluation_theme',
          'value' => $entity->id(),
        ],
      ],
    ];
    $questions = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_question', $params);

    if (!empty($questions)) {
      return [
        '#type' => 'link',
        '#title' => $this->t("List of questions"),
        '#url' => Url::fromRoute('entity.self_evaluation_theme.questions', ['self_evaluation_theme' => $entity->id()]),
      ];
    }
    return [
      '#type' => 'link',
      '#title' => $this->t("Add a question"),
      '#url' => Url::fromRoute('entity.self_evaluation_question.add_form', [], ['query' => ['self_evaluation_theme' => $entity->id()]]),
    ];
  }

  /**
   * Get data for thresholds cell for a theme.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Current Entity Theme.
   *
   * @return array
   *   Data.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getThresholdsCell(EntityInterface $entity): array {
    $params_thresholds = [
      'conditions' => [
        [
          'field' => 'self_evaluation_theme',
          'value' => $entity->id(),
        ],
      ],
      'sorts' => [
        [
          'field' => 'max_score',
        ],
      ],
    ];
    $thresholds = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_theme_threshold', $params_thresholds);
    if (!empty($thresholds)) {
      return [
        '#type' => 'link',
        '#title' => $this->t("List of thresholds"),
        '#url' => Url::fromRoute('entity.self_evaluation_theme.thresholds', ['self_evaluation_theme' => $entity->id()]),
      ];
    }
    return [
      '#type' => 'link',
      '#title' => $this->t("Add a Threshold"),
      '#url' => Url::fromRoute('entity.self_evaluation_theme_threshold.add_form', [], ['query' => ['self_evaluation_theme' => $entity->id()]]),
    ];
  }

}
