<?php

namespace Drupal\self_evaluation\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\self_evaluation\SelfEvaluationThemeInterface;
use Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for thresholds admin routes.
 */
class SelfEvaluationThresholdsController extends SelfEvaluationListController {

  /**
   * Constructs a new SelfEvaluationThresholdsController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type manager service.
   * @param \Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever
   *   Entity Retriever service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever) {
    $this->entityTypeManager = $entity_type_manager;
    $this->selfEvaluationEntityRetriever = $selfEvaluationEntityRetriever;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('self_evaluation.entity_retriever')
    );
  }

  /**
   * Return Threshold List for a theme.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationThemeInterface $self_evaluation_theme
   *   The current Theme Entity.
   *
   * @return array
   *   Array return.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function buildList(SelfEvaluationThemeInterface $self_evaluation_theme): array {
    $rows = [];
    $params = [
      'conditions' => [
        [
          'field' => 'self_evaluation_theme',
          'value' => $self_evaluation_theme->id(),
        ],
      ],
      'sorts' => [
        [
          'field' => 'max_score',
          'direction' => 'DESC',
        ],
      ],
    ];
    $thresholds = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_theme_threshold', $params);

    if (!empty($thresholds)) {
      foreach ($thresholds as $threshold) {
        $rows[] = $this->buildRow($threshold);
      }
    }
    $build = $this->render();

    $build['add']['#title'] = $this->t('Add Threshold');
    $build['add']['#url'] = Url::fromRoute('entity.self_evaluation_theme_threshold.add_form', [], ['query' => ['self_evaluation_theme' => $self_evaluation_theme->id()]]);
    $build['list']['#empty'] = $this->t('There are no threshold yet.');
    $build['list']['#rows'] = $rows;
    return $build;
  }

  /**
   * Builds the header row for the entity listing.
   *
   * @return array
   *   A renderable array structure of header strings.
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildHeader(): array {
    $header = [];
    $header['title'] = $this->t('Title');
    $header['max_score'] = $this->t('Maximum Score');
    return $header + parent::buildHeader();
  }

  /**
   * Builds a row for an entity in the entity listing.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for this row of the list.
   *
   * @return array
   *   A renderable array structure of fields for this entity.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildRow(EntityInterface $entity): array {
    $row = [];
    /** @var \Drupal\self_evaluation\SelfEvaluationThemeThresholdInterface $entity */
    $row['title']['data'] = $entity->label();
    $row['max_score']['data'] = $entity->getMaxScore();
    return $row + parent::buildRow($entity);
  }

}
