<?php

namespace Drupal\self_evaluation\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Template\Attribute;
use Drupal\self_evaluation\SelfEvaluationInterface;
use Drupal\self_evaluation\Services\SelfEvaluationDashboardManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Display a dashboard for user's self evaluation performed.
 *
 * @Block(
 *   id = "self_evaluation_dashboard_block",
 *   admin_label = @Translation("Self Evaluation Dashboard Block"),
 * )
 */
class SelfEvaluationDashboardBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * Entity Type Manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * User Manager Service.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * SelfEvaluation Entity Retriever service.
   *
   * @var \Drupal\self_evaluation\Services\SelfEvaluationDashboardManager
   */
  protected $selfEvaluationDashboardManager;

  /**
   * The route match service.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * Constructs a self evaluation dashboard block instance.
   *
   * @param array $configuration
   *   The plugin configuration, i.e. an array with configuration values keyed
   *   by configuration option name. The special key 'context' may be used to
   *   initialize the defined contexts by setting it to an array of context+
   *   values keyed by context names.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity Type Manager Service.
   * @param \Drupal\Core\Session\AccountProxyInterface $currentUser
   *   User manager service.
   * @param \Drupal\self_evaluation\Services\SelfEvaluationDashboardManager $selfEvaluationDashboardManager
   *   Self Evaluation Dashboard Manager.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager, AccountProxyInterface $currentUser, SelfEvaluationDashboardManager $selfEvaluationDashboardManager, RouteMatchInterface $route_match) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entityTypeManager;
    $this->currentUser = $currentUser;
    $this->selfEvaluationDashboardManager = $selfEvaluationDashboardManager;
    $this->routeMatch = $route_match;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('self_evaluation.dashboard_manager'),
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
        'self_evaluation' => NULL,
        'load_all_if_empty' => FALSE,
        'current_user_fallback' => FALSE,
      ] + parent::defaultConfiguration();
  }


  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $self_evaluations = NULL;
    if (!empty($this->configuration['self_evaluation'])) {
      $target_ids = array_column($this->configuration['self_evaluation'], 'target_id');
      $self_evaluations = $this->entityTypeManager->getStorage('self_evaluation')->loadMultiple($target_ids);
    }

    $form['self_evaluation'] = [
      '#type' => 'entity_autocomplete',
      '#title' => $this->t('Self Evaluation'),
      '#description' => $this->t('Select one or more self evaluations to be included in the dashboard.'),
      '#target_type' => 'self_evaluation',
      '#tags' => TRUE,
      '#size' => 120,
      '#default_value' => $self_evaluations,
    ];

    $form['load_all_if_empty'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Load all self evaluation if none is selected'),
      '#description' => $this->t('If none self evaluation is selected, then the dashboard will load all self evaluations available on the site.'),
      '#default_value' => $this->configuration['load_all_if_empty'],
    ];

    $form['current_user_fallback'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Fallback to current user'),
      '#description' => $this->t('The user for which you want get self evaluations will be loaded from the route. If no user found this option will fallback to the current user.'),
      '#default_value' => $this->configuration['current_user_fallback'],
    ];

    return parent::buildConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['self_evaluation'] = $form_state->getValue('self_evaluation');
    $this->configuration['load_all_if_empty'] = $form_state->getValue('load_all_if_empty');
    $this->configuration['current_user_fallback'] = $form_state->getValue('current_user_fallback');
    parent::submitConfigurationForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function build(): array {
    $dashboards = [];
    $build = [];
    $self_evaluations = [];
    $account = NULL;
    $user = $this->routeMatch->getParameter('user');
    if ($user instanceof AccountInterface) {
      $account = $user;
    }
    elseif ($this->configuration['current_user_fallback']) {
      $account = $this->currentUser;
    }
    if (!$account instanceof AccountInterface) {
      return $build;
    }

    if (!empty($this->configuration['self_evaluation'])) {
      $target_ids = array_column($this->configuration['self_evaluation'], 'target_id');
      $self_evaluations = $this->entityTypeManager->getStorage('self_evaluation')->loadMultiple($target_ids);
    }
    if (empty($self_evaluations)) {
      if ($this->configuration['load_all_if_empty']) {
        $self_evaluations = $this->entityTypeManager->getStorage('self_evaluation')->loadByProperties(['status' => TRUE]);
      }
      else {
        return $build;
      }
    }
    /** @var \Drupal\self_evaluation\Entity\SelfEvaluation $self_evaluation */
    foreach ($self_evaluations as $self_evaluation) {
      if (!$self_evaluation instanceof SelfEvaluationInterface) {
        continue;
      }
      if (!$self_evaluation->isEnabled()) {
        continue;
      }
      $self_evaluation_details = $this->selfEvaluationDashboardManager->getAnswersBySelfEvaluation($self_evaluation, $account);
      $dashboard = $this->selfEvaluationDashboardManager->getSelfEvaluationDashBoard($self_evaluation_details);
      $dashboards[] = $dashboard;
    }
    $attributes = new Attribute(['class' => ['dashboards__wrapper']]);
    $build = [
      '#theme' => 'self_evaluation_dashboards',
      '#wrapper_attributes' => $attributes,
      '#dashboards' => $dashboards,
    ];
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheTags() {
    $tags = parent::getCacheTags();
    return Cache::mergeTags($tags, ['self_evaluation_list']);
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheContexts() {
    $contexts = parent::getCacheContexts();
    if ($this->configuration['current_user_fallback']) {
      $contexts = Cache::mergeContexts($contexts, ['user']);
    }
    return Cache::mergeContexts($contexts, ['route']);
  }

}
