<?php

namespace Drupal\self_evaluation\Plugin\self_evaluation\FormStepsMode;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Template\Attribute;
use Drupal\self_evaluation\Entity\SelfEvaluationQuestion;
use Drupal\self_evaluation\FormStepsMode\FormStepsModeInterface;
use Drupal\self_evaluation\GroupmentsMode\GroupmentsModeManager;
use Drupal\self_evaluation\QuestionType\QuestionTypeManager;
use Drupal\self_evaluation\SelfEvaluationAnswerInterface;
use Drupal\self_evaluation\SelfEvaluationInterface;
use Drupal\self_evaluation\SelfEvaluationQuestionInterface;
use Drupal\self_evaluation\SelfEvaluationUserChoicesInterface;
use Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Abstract Class for FormStepsMode Plugin.
 */
abstract class FormStepsMode extends PluginBase implements FormStepsModeInterface, ContainerFactoryPluginInterface {

  /**
   * Entity type Manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $currentUser;

  /**
   * Current request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Question Type Manager.
   *
   * @var \Drupal\self_evaluation\QuestionType\QuestionTypeManager
   */
  protected $questionTypeManager;

  /**
   * Self Evaluation Entity Retriever Service.
   *
   * @var \Drupal\self_evaluation\Services\SelfEvaluationEntityRetriever
   */
  protected $selfEvaluationEntityRetriever;

  /**
   * Form step mode Instance.
   *
   * @var \Drupal\self_evaluation\QuestionType\QuestionTypeInterface
   */
  protected $questionTypeInstance;

  /**
   * Groupments mode Manager.
   *
   * @var \Drupal\self_evaluation\GroupmentsMode\GroupmentsModeManager
   */
  protected $groupmentsModeManager;

  /**
   * Current Self Evaluation.
   *
   * @var \Drupal\self_evaluation\SelfEvaluationInterface
   */
  protected $selfEvaluation;

  /**
   * Current Self Evaluation Answer.
   *
   * @var \Drupal\self_evaluation\SelfEvaluationAnswerInterface
   */
  protected $answer;

  /**
   * Current Step.
   *
   * @var int
   */
  protected $step;

  /**
   * Total Questions.
   *
   * @var int
   */
  protected $countQuestions;

  /**
   * Total Themes.
   *
   * @var int
   */
  protected $countThemes;

  /**
   * Total Steps.
   *
   * @var int
   */
  protected $countSteps;

  /**
   * Class constructor.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entityTypeManager, AccountProxyInterface $currentUser, RequestStack $requestStack, QuestionTypeManager $questionTypeManager, SelfEvaluationEntityRetriever $selfEvaluationEntityRetriever, GroupmentsModeManager $groupmentsModeManager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entityTypeManager;
    $this->currentUser = $currentUser;
    $this->requestStack = $requestStack;
    $this->questionTypeManager = $questionTypeManager;
    $this->selfEvaluationEntityRetriever = $selfEvaluationEntityRetriever;
    $this->groupmentsModeManager = $groupmentsModeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): FormStepsMode {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('current_user'),
      $container->get('request_stack'),
      $container->get('plugin.manager.self_evaluation.question_type'),
      $container->get('self_evaluation.entity_retriever'),
      $container->get('plugin.manager.self_evaluation.groupments_mode')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function description() {
    return $this->pluginDefinition['description'];
  }

  /**
   * {@inheritdoc}
   */
  public function init(SelfEvaluationInterface $self_evaluation, ?int $step, SelfEvaluationAnswerInterface $answer) {
    $this->selfEvaluation = $self_evaluation;
    $this->step = $step;
    $this->answer = $answer;
  }

  /**
   * {@inheritdoc}
   */
  public function getElements(): array {
    if ($this->step == 0) {
      return $this->buildGroupmentsType();
    }
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(FormStateInterface $form_state) {
    if ($this->step != 0 && $this->step == $this->countSteps) {
      $form_state->setRedirect('entity.self_evaluation.close', ['self_evaluation' => $this->selfEvaluation->id()]);
    }
    else {
      $form_state->setRedirect('entity.self_evaluation.form', [
        'self_evaluation' => $this->selfEvaluation->id(),
        'step' => $this->step + 1,
      ]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSteps(SelfEvaluationInterface $self_evaluation, SelfEvaluationAnswerInterface $answer): array {
    $this->selfEvaluation = $self_evaluation;
    $this->answer = $answer;
    return [];
  }

  /**
   * Return groupments type choices.
   *
   * @return array
   *   List of choices.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Component\Plugin\Exception\PluginException
   */
  public function buildGroupmentsType(): array {
    $form = $list_groupments_type = [];

    /** @var \Drupal\self_evaluation\Entity\SelfEvaluationGroupmentType $groupment_type */
    foreach ($this->selfEvaluation->getGroupmentTypes() as $groupment_type) {
      $options = [];
      $default_value = NULL;
      $params = [
        'conditions' => [
          [
            'field' => 'bundle',
            'value' => $groupment_type->id(),
          ],
        ],
      ];
      $groupments = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_groupment', $params);
      /** @var \Drupal\self_evaluation\SelfEvaluationGroupmentInterface $groupment */
      foreach ($groupments as $groupment) {
        $options[$groupment->id()] = $groupment->label();
      }
      foreach ($this->answer->getGroupments() as $groupment_answer) {
        if ($groupment_answer->bundle() == $groupment_type->id()) {
          $default_value = $groupment_answer->id();
        }
      }
      $list_groupments_type[$groupment_type->id()] = [
        'id' => $groupment_type->id(),
        'label' => $groupment_type->label(),
        'widget' => $groupment_type->get('widget'),
        'groupments' => $options,
        'default_value' => $default_value,
      ];
    }

    foreach ($list_groupments_type as $id => $groupment_type) {
      $groupments_instance = $this->groupmentsModeManager->createInstance($groupment_type['widget']);
      $form[$id] = $groupments_instance->render($groupment_type);
    }

    return [
      'groupment_types' => [
        '#theme' => 'self_evaluation_groupments_type_step',
        '#title' => $this->t('Categorize your answer'),
        '#self_evaluation' => $this->selfEvaluation,
        '#list_groupments_type' => $list_groupments_type,
        '#attributes' => new Attribute([
          'class' => [
            'step',
            'step--groupments_type',
          ],
        ]),
      ],
      'list_groupments_type' => $form,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildThemes(array $themes): array {
    $list_questions = [];
    foreach ($themes[$this->step]['questions'] as $question) {
      $list_questions[] = $this->buildQuestion($question, $question['number_question']);
    }
    $current_theme = $themes[$this->step]['theme'] ?? NULL;
    $attributes = new Attribute(['class' => ['step', 'step--theme']]);
    $current_attributes = new Attribute(['class' => ['current']]);
    $total_attributes = new Attribute(['class' => ['total']]);
    return [
      'step' => [
        '#theme' => 'self_evaluation_theme_step',
        '#title' => $this->t('Theme'),
        '#current' => $this->step,
        '#total' => $this->countThemes,
        '#current_theme' => $current_theme,
        '#attributes' => $attributes,
        '#current_attributes' => $current_attributes,
        '#total_attributes' => $total_attributes,
      ],
      'list_questions' => $list_questions,
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function buildQuestion(array $question, int $number_question): array {
    $options = $comment = $default_values = [];
    $unique_choice = NULL;
    /** @var \Drupal\self_evaluation\SelfEvaluationQuestionInterface $question_instance */
    $question_instance = $question['question'];
    $this->questionTypeInstance = $this->questionTypeManager->createInstance($question_instance->question_type->value);
    $user_choice = $this->getUserChoice($question_instance);
    if (!empty($user_choice)) {
      $choices_user = $user_choice->getSelfEvaluationQuestionChoices();
    }
    $comment_user = !empty($user_choice->comment) ? $user_choice->comment->value : NULL;
    foreach ($question['choices'] as $choice) {
      $informations_text = '';
      if ($choice->unique->value == '1') {
        $unique_choice = "choice-unique-{$choice->id()}";
        $informations_text = $this->t(
          "<span class='informations-unique informations-unique-@choice_id'>This answer is exclusive, others proposals have been disabled.</span>",
          ['@choice_id' => $choice->id()]
        );
      }
      $options[$choice->id()] = $choice->label() . $informations_text;
      if (!empty($choices_user) && in_array($choice, $choices_user)) {
        $default_values[] = $choice->id();
      }
    }

    $type = $this->questionTypeInstance->getType();
    $default_values = $this->questionTypeInstance->getDefaultValues($default_values);

    $choices = [
      '#type' => $type,
      '#options' => $options,
      '#default_value' => $default_values,
      '#attributes' => [
        'class' => [
          'choices',
          "choices-question-{$question_instance->id()}",
          $type == 'checkboxes' ? 'choices-multiple' : NULL,
          $unique_choice,
        ],
        'data-unique' => $unique_choice,
        'data-question-id' => $question_instance->id(),
      ],
    ];

    if ($question_instance->custom_answer->value == '1') {
      $comment = [
        '#type' => 'textarea',
        '#title' => $question_instance->custom_answer_title->value,
        '#required' => FALSE,
        '#default_value' => $comment_user,
      ];
    }
    $current_question = $question_instance;
    $current_theme = $question['theme'];
    $attributes = new Attribute(['class' => ['step', 'step--question']]);
    $current_attributes = new Attribute(['class' => ['current']]);
    $total_attributes = new Attribute(['class' => ['total']]);

    return [
      'step' => [
        '#theme' => 'self_evaluation_question_step',
        '#title' => $this->t('Question'),
        '#current' => $number_question,
        '#total' => $this->countQuestions,
        '#current_question' => $current_question,
        '#current_theme' => $current_theme,
        '#attributes' => $attributes,
        '#current_attributes' => $current_attributes,
        '#total_attributes' => $total_attributes,
      ],
      'theme' => $this->entityTypeManager->getViewBuilder('self_evaluation_theme')
        ->view($question['theme'], 'form'),
      'question' => $this->entityTypeManager->getViewBuilder('self_evaluation_question')
        ->view($question_instance, 'form'),
      "choices_{$question_instance->id()}" => $choices,
      "comment_{$question_instance->id()}" => $comment,
    ];
  }

  /**
   * Return the current User choice.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationQuestionInterface $question
   *   The current question.
   *
   * @return \Drupal\self_evaluation\SelfEvaluationUserChoicesInterface|null
   *   Array User choice (or user choice if unique).
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function getUserChoice(SelfEvaluationQuestionInterface $question): ?SelfEvaluationUserChoicesInterface {
    $params = [
      'conditions' => [
        [
          'field' => 'self_evaluation_question',
          'value' => $question->id(),
        ],
        [
          'field' => 'self_evaluation_answer',
          'value' => $this->answer->id(),
        ],
      ],
    ];
    $user_choice = $this->selfEvaluationEntityRetriever->getEntities('self_evaluation_user_choices', $params, 1);
    return !empty($user_choice) ? current($user_choice) : NULL;
  }

  /**
   * Save the current answer.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param \Drupal\self_evaluation\Entity\SelfEvaluationQuestion $question
   *   The current question.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function saveAnswerQuestion(FormStateInterface $form_state, SelfEvaluationQuestion $question) {
    $choices = [];
    $comment = NULL;
    $form_choices = $form_state->getValue("choices_{$question->id()}");

    if (is_array($form_choices)) {
      foreach ($form_choices as $choice_id) {
        if (!empty($choice_id)) {
          /** @var \Drupal\self_evaluation\SelfEvaluationQuestionChoiceInterface $choice */
          $choice = $this->entityTypeManager->getStorage('self_evaluation_question_choice')
            ->load($choice_id);
          if ($choice->isUnique()) {
            $choices = [];
            $choices[] = $choice;
            break;
          }
          $choices[] = $choice;
        }
      }
    }
    else {
      $choices[] = $this->entityTypeManager->getStorage('self_evaluation_question_choice')
        ->load($form_choices);
    }
    if (!empty($form_state->getValue("comment_{$question->id()}"))) {
      $comment = $form_state->getValue("comment_{$question->id()}");
    }
    $this->setUserChoice($question, $choices, $comment);
    $this->answer->set('self_evaluation_question', $question);
    $this->answer->save();
  }

  /**
   * Save answer groupments.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function saveAnswerGroupments(FormStateInterface $form_state) {
    $groupments = [];

    $groupment_storage = $this->entityTypeManager->getStorage('self_evaluation_groupment');
    /** @var \Drupal\self_evaluation\Entity\SelfEvaluationGroupmentType $groupment_type */
    foreach ($this->selfEvaluation->getGroupmentTypes() as $groupment_type) {
      if (!empty($form_state->getValue("groupment_type_{$groupment_type->id()}"))) {
        $groupments[] = $groupment_storage->load($form_state->getValue("groupment_type_{$groupment_type->id()}"));
      }
    }

    $this->answer->set('self_evaluation_groupments', $groupments);
    $this->answer->save();
  }

  /**
   * Return the current self evaluation user choice entity.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationQuestionInterface $question
   *   The current question.
   * @param array $choices
   *   Array of Choices.
   * @param string|null $comment
   *   The comment associated.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  protected function setUserChoice(SelfEvaluationQuestionInterface $question, array $choices, string $comment = NULL) {
    $user_choice = $this->getUserChoice($question);
    if (empty($user_choice)) {
      $user_choice = $this->entityTypeManager->getStorage('self_evaluation_user_choices')->create([
        'self_evaluation_answer' => $this->answer,
        'self_evaluation_question' => $question,
        'self_evaluation_question_choices' => $choices,
        'comment' => $comment,
      ]);
    }
    else {
      $user_choice->set('self_evaluation_question_choices', $choices);
      $user_choice->set('comment', $comment);
    }
    $user_choice->save();
  }

}
