<?php

namespace Drupal\Tests\self_evaluation\Traits;

use Drupal\Core\Entity\EntityInterface;
use Drupal\self_evaluation\Entity\SelfEvaluation;
use Drupal\self_evaluation\Entity\SelfEvaluationQuestion;
use Drupal\self_evaluation\Entity\SelfEvaluationQuestionChoice;
use Drupal\self_evaluation\Entity\SelfEvaluationTheme;
use Drupal\self_evaluation\Entity\SelfEvaluationThemeThreshold;
use Drupal\self_evaluation\SelfEvaluationInterface;
use Drupal\Tests\Core\Session\AccountProxyTest;
use Drupal\user\Entity\User;

/**
 * Provides methods to values for entity type.
 *
 * This trait is meant to be used only by test classes.
 */
trait SelfEvaluationTrait {

  private function createSelfEvaluationEntity($form_steps_mode, $note_type, $chart_type, $customEntity = NULL) {

    $values = [
      'title[0][value]' => $this->randomMachineName(25),
      'description' => $this->randomMachineName(32),
      'start_button_label' => $this->randomMachineName(10),
      'shareable_results' => 1,
      'publishable_results' => 1,
      'note_type' => $note_type,
      'form_steps_mode' => $form_steps_mode,
      'chart_type' => [$chart_type],
    ];
    $self_evaluation = SelfEvaluation::create($values);
    $self_evaluation->save();

    if (!empty($customEntity)) {
      $this->$customEntity = $self_evaluation;
    }
    else {
      $this->selfEvaluation = $self_evaluation;
    }
  }

  /**
   * Create an entity from type.
   *
   * @param string $type
   *   Current type.
   * @param string|null $customEntity
   *   Custom Entity Variable.
   * @param string|null $customParent
   *   Custom Parent Variable.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  private function createEntity(string $type, string $customEntity = NULL, string $customParent = NULL) {
    if ($type == 'self_evaluation_question' || $type == 'self_evaluation_question_choice') {
      $values = [
        'title[0][value]' => $this->randomMachineName(25),
      ];
    }
    else {
      $values = [
        'name[0][value]' => $this->randomMachineName(25),
      ];
    }

    switch ($type) {
      case 'self_evaluation_theme':
        $self_evaluation_theme = SelfEvaluationTheme::create($this->initParams($values, $type, $customParent));
        $self_evaluation_theme->save();

        if (!empty($customEntity)) {
          $this->$customEntity = $self_evaluation_theme;
        }
        else {
          $this->selfEvaluationTheme = $self_evaluation_theme;
        }
        break;

      case 'self_evaluation_theme_threshold':
        $self_evaluation_theme_threshold = SelfEvaluationThemeThreshold::create($this->initParams($values, $type, $customParent));
        $self_evaluation_theme_threshold->save();

        if (!empty($customEntity)) {
          $this->$customEntity = $self_evaluation_theme_threshold;
        }
        else {
          $this->selfEvaluationThemeThreshold = $self_evaluation_theme_threshold;
        }

        break;

      case 'self_evaluation_question':
        $self_evaluation_question = SelfEvaluationQuestion::create($this->initParams($values, $type, $customParent));
        $self_evaluation_question->save();

        if (!empty($customEntity)) {
          $this->$customEntity = $self_evaluation_question;
        }
        else {
          $this->selfEvaluationQuestion = $self_evaluation_question;
        }
        break;

      case 'self_evaluation_question_choice':
        $self_evaluation_question_choice = SelfEvaluationQuestionChoice::create($this->initParams($values, $type, $customParent));
        $self_evaluation_question_choice->save();

        if (!empty($customEntity)) {
          $this->$customEntity = $self_evaluation_question_choice;
        }
        else {
          $this->selfEvaluationQuestionChoice = $self_evaluation_question_choice;
        }
        break;
    }
  }

  /**
   * Get an entity from the database based on its title.
   *
   * @param string|\Drupal\Component\Render\MarkupInterface $title
   *   A node title, usually generated by $this->randomMachineName().
   * @param string $type
   *   Entity type.
   * @param bool $reset
   *   (optional) Whether to reset the entity cache.
   *
   * @return \Drupal\Core\Entity\EntityInterface
   *   A node entity matching $title.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getEntityByTitle($title, string $type, bool $reset = FALSE): EntityInterface {
    $etm = \Drupal::entityTypeManager();
    if ($reset) {
      $etm->getStorage($type)->resetCache();
    }
    // Cast MarkupInterface objects to string.
    $title = (string) $title;

    $title_field = $type == 'self_evaluation' || $type == 'self_evaluation_question' || $type == 'self_evaluation_question_choice' ? 'title' : 'name';

    $entities = $etm
      ->getStorage($type)
      ->loadByProperties([$title_field => $title]);
    // Load the first entity returned from the database.
    return reset($entities);
  }

  /**
   * Retrieve last self evaluation answer.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationInterface $self_evaluation
   *   A self evaluation.
   * @param \Drupal\user\Entity\User $user
   *   Current user.
   *
   * @return false|mixed
   *   Last self evaluation answer.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getAnswer(SelfEvaluationInterface $self_evaluation, User $user) {
    $etm = \Drupal::entityTypeManager();

    $entities = $etm
      ->getStorage('self_evaluation_answer')
      ->loadByProperties(
        [
          'self_evaluation' => $self_evaluation->id(),
          'uid' => $user->id(),
        ]
      );
    // Load the first entity returned from the database.
    return reset($entities);
  }

}
